#include <curl/curl.h>
#include <sstream>
#include <string>
#include <iostream>
#include <vector>
#include "histogram.h"
#include "text.h"
#include "svg.h"

struct Input
{
    std::vector<double> numbers;
    size_t bin_count{};
};

Input input_data(std::istream& in, bool prompt)
{
    Input data;
    size_t number_count;
    if(prompt)
    {
        cerr << "Enter the number of elements: ";
    }
    in >> number_count;
    data.numbers.resize(number_count);
    if(prompt)
    {
        cerr << "\nEnter " << number_count << " elements:" << endl;
    }
    for (size_t i = 0; i < number_count; i++)
    {
        in >> data.numbers[i];
    }
    if(prompt)
    {
        cerr << "Enter the number of bins: ";
    }
    in >> data.bin_count;
    return data;
}
size_t write_data(void* items, size_t item_size, size_t item_count, void* ctx){
    stringstream* buffer = reinterpret_cast<stringstream*>(ctx);
    size_t data_size = item_size * item_count;
    buffer->write(reinterpret_cast<const char*>(items), data_size);
    return data_size;
}
Input download(const string& address)
{
    stringstream buffer;
    CURL* curl = curl_easy_init();
    if (curl)
    {
        curl_easy_setopt(curl, CURLOPT_URL, address.c_str());
        curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION, write_data);
        curl_easy_setopt(curl, CURLOPT_WRITEDATA, &buffer);
        CURLcode res = curl_easy_perform(curl);
        curl_easy_cleanup(curl);
        if (res != CURLE_OK)
        {
            cerr << "cURL error: " << curl_easy_strerror(res) << endl;
            exit(1);
        }
        return input_data(buffer, false);
    }
}
int
main(int argc, char* argv[]) {
    Input input;
    if (argc > 1) {
        input = download(argv[1]);
    } else {
        input = input_data(cin, true);
    }
    auto bins = make_histogram(input.bin_count, input.numbers);
    show_histogram_svg(bins);
}


